/*======================================================================

  This file is part of the elastix software.

  Copyright (c) University Medical Center Utrecht. All rights reserved.
  See src/CopyrightElastix.txt or http://elastix.isi.uu.nl/legal.php for
  details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE. See the above copyright notices for more information.

======================================================================*/

#ifndef _itkErodeMaskImageFilter2_txx
#define _itkErodeMaskImageFilter2_txx

#include "itkErodeMaskImageFilter2.h"
#include "itkBinaryErodeImageFilter.h"
#include "itkBinaryBallStructuringElement.h"


namespace itk
{


/** 
 * ************* Constructor *******************
 */

template< class TImage >
ErodeMaskImageFilter2< TImage >
::ErodeMaskImageFilter2()
{
  this->m_IsMovingMask = false;
  this->m_ResolutionLevel = 0;

  ScheduleType defaultSchedule( 1, InputImageDimension );
  defaultSchedule.Fill( NumericTraits< unsigned int >::One );
  this->m_Schedule = defaultSchedule;

} // end Constructor


/** 
 * ************* GenerateData *******************
 */

template< class TImage >
void
ErodeMaskImageFilter2< TImage >
::GenerateData( void )
{
  /** Typedefs. */
  typedef BinaryBallStructuringElement<
    InputPixelType, InputImageDimension >               StructuringElementType;
  typedef typename StructuringElementType::RadiusType   RadiusType;
  typedef BinaryErodeImageFilter<
    InputImageType, OutputImageType,
    StructuringElementType >                            ErodeFilterType;

  /** Get some handles. */
  OutputImagePointer output = this->GetOutput();
  unsigned int level = this->GetResolutionLevel();

  /** Create a structuring element with the correct radius. */
  RadiusType                radiusarray;
  StructuringElementType    S_ball;
  unsigned long             radius;
  unsigned int              schedule;
  for ( unsigned int i = 0; i < InputImageDimension; ++i )
  {
    schedule = this->GetSchedule()[ level ][ i ];
    if ( ! this->GetIsMovingMask() )
    {
      radius = static_cast<unsigned long>( schedule + 1 );
    }
    else
    {
      radius = static_cast<unsigned long>( 2 * schedule + 1 );
    }
    radiusarray.SetElement( i, radius );
  }
  S_ball.SetRadius( radiusarray );
  S_ball.CreateStructuringElement();

  /** Create and run the erosion filter. */
  typename ErodeFilterType::Pointer erosion = ErodeFilterType::New();
  erosion->SetForegroundValue( NumericTraits<InputPixelType>::One );
  erosion->SetBackgroundValue( NumericTraits<InputPixelType>::Zero );
  erosion->SetKernel( S_ball );
  erosion->SetInput( this->GetInput() );
  erosion->Update();

  /** Graft the output of the mini-pipeline back onto the filter's output.
   * this copies back the region ivars and meta-data.
   */
  this->GraftOutput( erosion->GetOutput() );

} // end GenerateData()

} // end namespace itk

#endif
